#!/usr/bin/ruby -w

require 'socket'
require 'thread'

class ChatServer
	
	def initialize(host, port)
		@server = TCPServer.new(host, port)
		@semaphore = Mutex.new
		@clients = []
	end

	def serve
		while socket = @server.accept
			#	minde csatlakozott sockethez uj szalat rendel
			client = ClientThread.new(socket)
			client.on_recieve do |c,l|
				distribute(c,l)
			end
			add_client(client)
			client.listen
		end
	end

	def distribute(client, line)
		@semaphore.synchronize do #	vedett szakasz lezarasa
			@clients.each do |c|
				c.send(line) if c != client
			end
		end
	end

	def add_client(client)
		@semaphore.synchronize do
			@clients.push client
		end
		client.on_terminate do |c|
			remove_client c
		end
	end

	def remove_client(client)
		@semaphore.synchronize do
			@clients.delete client
		end
	end
end

class ClientThread

	def initialize(socket)
		@socket = socket
		@on_recieve = @on_terminate = nil
	end

	def listen
		@listen_thread = Thread.new do
			while line = @socket.gets
				break if /^QUIT/ =~ line
				@on_recieve.call(self, line) if @on_recieve	#	block hivasa
			end
			@on_terminate.call(self) if @on_terminate
			@socket.close
		end
	end

	def send(line)
		@socket.puts(line)
	end

	def on_recieve(&on_recieve)
		@on_recieve = on_recieve
	end

	def on_terminate(&on_terminate)
		@on_terminate = on_terminate
	end
end

host = ARGV[0] || 'localhost'
port = ARGV[1] || 1111
cs = ChatServer.new(host, port)
cs.serve
